#![no_std]
#![no_main]

#[macro_use]
extern crate user_lib;

use core::ptr::slice_from_raw_parts_mut;
use user_lib::sys_set_break;

#[no_mangle]
fn main() -> i32 {
    println!("Test set break start...");
    const PAGE_SIZE: usize = 0x1000;
    
    let origin_break = sys_set_break(0);
    println!("Origin break address is {:x}.", origin_break);
    
    let break_address = sys_set_break(PAGE_SIZE as i32);
    if origin_break != break_address {
        return -1;
    }
    
    let break_address = sys_set_break(0);
    println!("Now break address is {:x}.", break_address);
    
    println!("Try to write into allocated page.");
    let new_page = unsafe {
        &mut *slice_from_raw_parts_mut(origin_break as usize as *const u8 as *mut u8, PAGE_SIZE)
    };
    for pos in 0..PAGE_SIZE {
        new_page[pos] = 0;
    }
    println!("Write successfully.");
    
    sys_set_break(PAGE_SIZE as i32 * 10);
    let break_address = sys_set_break(0);
    println!("Allocated 10 more pages, break address is {:x}.", break_address);
    
    sys_set_break(PAGE_SIZE as i32 * -11);
    let break_address = sys_set_break(0);
    println!("Deallocated all 11 pages, break address is {:x}.", break_address);
    
    println!("Try to deallocated one more page, which should fail.");
    let result = sys_set_break(PAGE_SIZE as i32 * -1);
    if result == -1 {
        println!("But succeed, exit!");
        return -1;
    }
    
    println!("Try to write into deallocated pages, which should make this application killed by kernel.");
    for pos in 0..PAGE_SIZE {
        new_page[pos] = 0;
    }
    
    0
}